<?php

namespace Drupal\user_referral\Element;

use Drupal\Core\Render\Element\FormElement;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Drupal\user_referral\Entity\UserReferralType;
use Drupal\Core\Entity\Element\EntityAutocomplete;
use Drupal\Component\Utility\Html;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Render\Element;

/**
 * Provides a referrer form element.
 *
 * @FormElement("referrer")
 */
class Referrer extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#default_value' => NULL,
      '#user_title' => NULL,
      '#type_title' => NULL,
      '#process' => [
        [$class, 'processReferrer'],
        [$class, 'processGroup'],
      ],
      '#pre_render' => [
        // [$class, 'groupElements'],
        [$class, 'preRenderGroup'],
      ],
      '#after_build' => [
        // [$class, 'clearValues'],
      ],
      '#attached' => [
        // 'library' => ['address/form'],
      ],
      '#theme_wrappers' => ['fieldset'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if (is_array($input) && !isset($input['referrer'])) {
      $input = NULL;
    }
    if (is_array($input)) {
      if (empty($input['referrer']) && $element['#required']) {
        $input['referrer'] = $element['#default_value']['referrer'];
      }
    }
    else {
      $input = $element['#default_value'];
    }

    if (!empty($input['referrer'])) {
      if (is_string($input['referrer'])) {
        if (preg_match('/\((\d+)\)/', $input['referrer'], $matches)) {
          $input['referrer'] = User::load($matches[1]);
        }
        else if (preg_match('/^\d+$/', $input['referrer'], $matches)) {
          $input['referrer'] = User::load($matches[0]);
        }
      }
      else if (is_int($input['referrer'])) {
        $input['referrer'] = User::load($input['referrer']);
      }
    }
    return $input;
  }

  /**
   * Expands a referrer element type referrer user and/or type elements.
   *
   * @param array $element
   *   The form element whose value is being processed.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form structure.
   *
   * @return array
   *   The form element whose value has been processed.
   *
   */
  public static function processReferrer(&$element, FormStateInterface $form_state, &$complete_form) {
    // The value callback has populated the #value array.
    $value = !empty($element['#value']) ? $element['#value'] : NULL;

    $element['#tree'] = TRUE;

    // Generate a unique wrapper HTML ID.
    $ajax_wrapper_id = Html::getUniqueId('ajax-wrapper');

    $referrer_value = NULL;
    if (!empty($value) && is_object($value['referrer'])) {
      $referrer_value = EntityAutocomplete::getEntityLabels([$value['referrer']]);
    }

    $element['referrer'] = [
      '#type' => 'entity_autocomplete',
      '#selection_handler' => 'user_referral:referrer_user_selection',
      '#target_type' => 'user',
      '#title' => $element['#user_title'] ?? t('User'),
      '#value' => $referrer_value,
      '#required' => $element['#required'],
      '#ajax' => [
        'callback' => [get_called_class(), 'refreshReferralTypeField'],
        // 'disable-refocus' => FALSE, // Or TRUE to prevent re-focusing on the triggering element.
        'event' => 'change',
        'wrapper' => $ajax_wrapper_id , // This element is updated with this AJAX callback.
        'progress' => [
          'type' => 'throbber',
          'message' => t('Fetching applicable referral types...'),
        ],
        'options' => [
          'query' => [
            'element_parents' => implode('/', $element['#array_parents']),
          ],
        ],
      ]
    ];

    $referral_type_options = [];
    if (!empty($value['referrer']) && $value['referrer']) {
      foreach (UserReferralType::loadMultiple() as $referral_type_id => $referral_type) {
        $user_roles = $value['referrer']->getRoles();
        $referral_type_roles = $referral_type->getRoles();
        if (!empty(array_intersect($referral_type_roles, $user_roles))) {
          $referral_type_options[$referral_type_id] = $referral_type->label();
        }
      }
    }

    if (empty($referral_type_options) || count($referral_type_options) < 2 ) {
      $type_value = NULL;
      if (count($referral_type_options) == 1) {
        // Force select one and ony one Referral Type option.
        $referral_type_options_keys = array_keys($referral_type_options);
        $type_value = reset($referral_type_options_keys);
      }
      $element['type'] = [
        '#type' => 'value',
        '#value' => $type_value,
      ];
    }
    else {
      $element['type'] = [
        '#type' => 'radios',
        '#title' => $element['#type_title'] ?? t('Type'),
        '#default_value' => $value['type'],
        '#value' => $value['type'],
        '#options' => $referral_type_options,
        '#required' => $element['#required'],
      ];
    }

    // Prefix and suffix used for Ajax replacement.
    $element['#prefix'] = '<div id="' . $ajax_wrapper_id . '">';
    $element['#suffix'] = '</div>';

    return $element;
  }

  /**
   * Idea taked from \Drupal\file\Element\ManagedFile::uploadAjaxCallback()
   *
   * @param array $form
   *  The form array
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *  The form state object
   * @param \Symfony\Component\HttpFoundation\Request $request
   *  The current request for the ajax
   */
  public static function refreshReferralTypeField(array &$form, FormStateInterface $form_state, Request $request) {
    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = \Drupal::service('renderer');

    $form_parents = explode('/', $request->query->get('element_parents'));

    // Sanitize form parents before using them.
    $form_parents = array_filter($form_parents, [Element::class, 'child']);

    // Retrieve the element to be rendered.
    $form = NestedArray::getValue($form, $form_parents);
    return $form;
  }
}
