<?php

namespace Drupal\user_referral\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Random;
use Drupal\Core\TypedData\DataDefinitionInterface;
use Drupal\Core\TypedData\TypedDataInterface;
use Drupal\Core\Entity\FieldableEntityInterface;

/**
 * Plugin implementation of the 'telephone' field type.
 *
 * @FieldType(
 *   id = "referral_id",
 *   label = @Translation("Referral ID"),
 *   description = @Translation("This field stores Referral ID in database and ensure it is unique among users in the site."),
 *   category = @Translation("Referral"),
 *   default_widget = "referral_id_textfield",
 *   default_formatter = "basic_string",
 *   cardinality = 1
 * )
 */
class ReferralID extends FieldItemBase {


  public function __construct(DataDefinitionInterface $definition, $name = NULL, TypedDataInterface $parent = NULL) {
    $fieldDefinition = $definition->getFieldDefinition();
    // TODO: Make sure it correct way to set the callback
    $fieldDefinition->set('default_value_callback',[$this, 'defaultValue']);
    $definition->setFieldDefinition($fieldDefinition);
    parent::__construct($definition, $name, $parent);
  }

  /**
   * Dummy 'default value' callback.
   * This is used to hide default value widget in field settigs page.
   */
  function defaultValue(FieldableEntityInterface $entity, FieldDefinitionInterface $definition) {
    return [['value' => NULL]];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'max_length' => 255,
      'space_allowed' => FALSE,
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar',
          'length' => 256,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Referral ID'))
      ->setRequired(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('value')->getValue();
    return $value === NULL || $value === '';
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();
    $constraints = parent::getConstraints();

    if ($max_length = $this->getSetting('max_length')) {
      $constraints[] = $constraint_manager->create('ComplexData', [
        'value' => [
          'Length' => [
            'max' => $max_length,
            'maxMessage' => t('%name: the referral ID may not be longer than @max characters.', ['%name' => $this->getFieldDefinition()->getLabel(), '@max' => $max_length]),
          ],
        ],
      ]);
    }

    if (!($space_allowed = $this->getSetting('space_allowed'))) {
      $constraints[] = $constraint_manager->create('SpaceNotAllowed', []);
    }

    $constraints[] = $constraint_manager->create('UniqueReferralID', []);

    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $random = new Random();
    $value = $random->string(100, TRUE);
    $values = [
      'value' => $value,
    ];
    return $values;
  }

  /**
   * 
   */
  public function first() {
    // return $this->get('value')->getValue();
    return $this->get('value');
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $element = [];

    $element['max_length'] = [
      '#type' => 'number',
      '#title' => t('Maximum length'),
      '#default_value' => $this->getSetting('max_length'),
      '#required' => TRUE,
      '#description' => t('The maximum length of the field in characters.'),
      '#min' => 1,
      '#disabled' => $has_data,
    ];

    $element['space_allowed'] = [
      '#type' => 'checkbox',
      '#title' => t('Space characters are allowed'),
      '#default_value' => $this->getSetting('space_allowed'),
      '#description' => t('Enable if space characters are allowed in value.'),
      '#disabled' => $has_data,
    ];
    $element += parent::storageSettingsForm($form, $form_state, $has_data);

    return $element;
  }
}
