<?php

namespace Drupal\user_referral\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Render\Element;
use Drupal\user_referral\Entity\UserReferralType;

/**
 * Plugin implementation of the 'referral_id_textfield' widget.
 *
 * @FieldWidget(
 *   id = "referral_id_textfield",
 *   label = @Translation("Referral ID"),
 *   field_types = {
 *     "referral_id"
 *   }
 * )
 */
class ReferralIDTextfieldWidget extends WidgetBase implements  ContainerFactoryPluginInterface{

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ElementInfoManagerInterface $element_info) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->elementInfo = $element_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($plugin_id, $plugin_definition, $configuration['field_definition'], $configuration['settings'], $configuration['third_party_settings'], $container->get('element_info'));
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'size' => 60,
      'placeholder' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['size'] = [
      '#type' => 'number',
      '#title' => t('Size of textfield'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#min' => 1,
    ];
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary[] = t('Textfield size: @size', ['@size' => $this->getSetting('size')]);
    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = t('Placeholder: @placeholder', ['@placeholder' => $placeholder]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function form(FieldItemListInterface $items, array &$form, FormStateInterface $form_state, $get_delta = NULL) {
    if ($this->isDefaultValueWidget($form_state)) {
      // Not show show the widget to set default value.
      return [];
    }
    return parent::form($items, $form, $form_state, $get_delta);
  }


  /**
   * {@inheritdoc}
   */
  protected function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {
    $elements = parent::formMultipleElements($items, $form, $form_state);
    $elements['#process'] = [[get_class($this), 'processMultiple']];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $element['value'] = $element + [
      '#type' => 'textfield',
      '#default_value' => isset($items[$delta]->value) ? $items[$delta]->value : NULL,
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#maxlength' => $this->getFieldSetting('max_length'),
      '#attributes' => ['class' => ['js-text-full', 'text-full']],
      '#element_validate' => [[$this, 'validateFormElement']],
    ];

    $element['existing_value'] = [
      '#type' => 'value',
      '#value' => isset($items[$delta]->value) ? $items[$delta]->value : NULL,
    ];

    $element['button'] = [
      '#type' => 'button',
      '#value' => $this->t('Check availabiltiy'),
      '#ajax' => [
        'callback' => [$this, 'checkAvailability'],
        // 'disable-refocus' => FALSE, // Or TRUE to prevent re-focusing on the triggering element.
        'event' => 'click',
        // 'wrapper' => '', // Wrapper will be populated by ::processMultiple()
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Checking availability...'),
        ],
      ],
    ];

    return $element;
  }

  /**
   * Form element validation handler for URL alias form element.
   *
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function validateFormElement(array &$element, FormStateInterface $form_state) {
    $input_exists = FALSE;
    $input = NestedArray::getValue($form_state->getValues(), $element['#parents'], $input_exists);
    $existign_value_parents = array_merge(array_slice($element['#parents'], 0, -1), ['existing_value']);
    $existing_value = NestedArray::getValue($form_state->getValues(), $existign_value_parents);
    if ($input_exists && (empty($existing_value) || $input != $existing_value)) {
      $entity = $form_state->getformObject()->getEntity();

      $valid = $this->isValid($input, $entity);
      if (!$valid) {
        $form_state->setError($element, $this->t('Referral ID value %value is not available. Please enter a differrent value.', ['%value' => $input]));
      }
    }
  }

  /**
   * Checks if give value is valid.
   * 
   * @param string $value
   * @return boolean
   */
  public function isValid($value, $entity) {
    $space_allowed = $this->getSetting('space_allowed');
    $valid = TRUE;

    if (!$space_allowed) {
      $pattern = '/[a-zA-Z0-9]+/';
    }
    else {
      $pattern = '/[a-zA-Z0-9\s]+/';
    }

    $valid = (bool) preg_match($pattern, $value);

    if ($valid) {
      $field_name = $this->fieldDefinition->getName();
      $entity_type_id = $entity->getEntityTypeId();
      $id_key = $entity->getEntityType()->getKey('id');

      $query = \Drupal::entityQuery($entity_type_id);

      $entity_id = $entity->id();
      // Using isset() instead of !empty() as 0 and '0' are valid ID values for
      // entity types using string IDs.
      if (isset($entity_id)) {
        $query->condition($id_key, $entity_id, '<>');
      }

      $valid = !(bool) $query
        ->condition($field_name, $value)
        ->range(0, 1)
        ->count()
        ->execute();
    }

    if ($valid) {
      $reserved_referral_ids = \Drupal::configFactory()->get('user_referral.settings')->get('reserved_referral_ids');
      $list = explode("\n", $reserved_referral_ids);

      // Not allow to use referral type name as any referral ID.
      $list = array_merge($list, array_keys(UserReferralType::referralTypeOptions()));

      $list = array_map('trim', $list);
      $list = array_map('strtolower', $list);
      $list = array_filter($list, 'strlen');
      $valid = !in_array(strtolower($value), $list);
    }
    if ($valid) {
      // Check if no primary path exists corresponding to the entered  value.
      $valid  = !((bool) \Drupal::service('path.validator')->getUrlIfValidWithoutAccessCheck($value));
    }
    return $valid;
  }

  public function checkAvailability(array &$form, FormStateInterface $form_state) {
    $element = $form_state->getTriggeringElement();
    $parents = array_slice($element['#array_parents'], 0, -2);
    $wrapper_element = NestedArray::getValue($form, $parents);
    $value_element_parents = array_merge($parents, [0, 'value']);
    $value_element = NestedArray::getValue($form, $value_element_parents);

    $existign_value_parents = array_merge($parents, [0, 'existing_value']);
    $existing_value_element = NestedArray::getValue($form, $existign_value_parents);

    $value = $value_element['#value'];
    $existing_value = $existing_value_element['#value'];

    $entity = $form_state->getformObject()->getEntity();

    $available = (!empty($existing_value) && $value == $existing_value ) ? TRUE : $this->isValid($value, $entity);

    
    if (!$available) {
      $wrapper_element[0]['availability']['#markup'] = '<span class="ur-ri-not-available">' . $this->t('Not available') . '</span>';
    }
    else {
      $wrapper_element[0]['availability']['#markup'] = '<span class="ur-ri-not-available">' . $this->t('Available') . '</span>';
    }
    return $wrapper_element;
  }

  /**
   * Form API callback: Processes a group  field elements.
   *
   * Adds the wrapper for Ajax replacements
   *
   * This method on is assigned as a #process callback in formMultipleElements()
   * method.
   */
  public static function processMultiple($element, FormStateInterface $form_state, $form) {
    $element_children = Element::children($element, TRUE);

    $new_wrapper = $element['#id'] . '-ajax-wrapper';


    foreach ($element_children as $delta => $key) {
      $element[$key]['button']['#ajax']['wrapper'] = $new_wrapper;
    }

    // Add a new wrapper around all the elements for Ajax replacement.
    $element['#prefix'] = '<div id="' . $new_wrapper . '">';
    $element['#suffix'] = '</div>';

    return $element;
  }

}
