<?php

namespace Drupal\user_referral\Tests\Functional;

use Drupal\user\Entity\User;
use Drupal\Tests\user_referral\Traits\UserReferralTypeTestTrait;
use Drupal\user_referral\Entity\UserReferralType;
use Drupal\Tests\BrowserTestBase;

/**
 * Test basic functionality of user_referral module.
 *
 * @group user_referral
 */
class UserReferralBasicTest extends BrowserTestBase {

  use UserReferralTypeTestTrait {
    createUserReferralType as drupalCreateUserReferralType;
  }

  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'user_referral',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $permissions = [
      'access administration pages',
      'administer modules',
      'administer site configuration',
      'administer users',
      'administer user referral types',
    ];

    $this->consultant_role = $this->drupalCreateRole([], 'consultant');
    $this->referral_partner_role = $this->drupalCreateRole([], 'referral partner');

    $this->consultant_referral_type = $this->drupalCreateUserReferralType([
      'type' => 'consultant',
      'name' => 'Consultant',
      'referral_field' => 'field_referral_id',
      'roles' => ['consultant'],
      'url_alias_pattern' => '/[user_referral_type:referral_field_value]',
      'reset_cookie_lifetime_for_same_referrer' => TRUE,
      'change_referrer_in_cookie' => TRUE,
    ]);

    $this->referral_partner_referral_type = $this->drupalCreateUserReferralType([
      'type' => 'referralpartner',
      'name' => 'Referral Partner',
      'referral_field' => 'field_referral_id',
      'roles' => ['referral partner'],
      'url_alias_pattern' => '/[user_referral_type:referral_field_value]/[user_referral_type:type]',
      'reset_cookie_lifetime_for_same_referrer' => TRUE,
      'change_referrer_in_cookie' => TRUE,
    ]);

    $this->consultant_referrer_1 = $this->drupalCreateUser([], 'creferrer1', FALSE, ['field_referral_id' => 'creferrer1', 'roles' => ['consultant']]);
    $this->consultant_referrer_2 = $this->drupalCreateUser([], 'creferrer2', FALSE, ['field_referral_id' => 'creferrer2', 'roles' => ['consultant']]);
    $this->referral_partner_referrer = $this->drupalCreateUser([], 'rpreferrer2', FALSE, ['field_referral_id' => 'rpreferrer2', 'roles' => ['referral partner']]);
    
    // Ensure referral link aliases are created.
    $this->consultant_referral_type->getReferralLink($this->consultant_referrer_1);
    $this->consultant_referral_type->getReferralLink($this->consultant_referrer_2);
    $this->referral_partner_referral_type->getReferralLink($this->referral_partner_referrer);

    // User to set up realname.
    $this->admin_user = $this->drupalCreateUser($permissions);
  }

  /**
   * Test realname configuration.
   */
  public function testReferralTypesManagement() {

    $this->drupalLogin($this->admin_user);

    // Requires 'administer user referral types' permission.
    $this->drupalGet('admin/structure/user-referral-types');
    $this->assertSession()->statusCodeEquals(200);
    $assert_session = $this->assertSession();
    $assert_session->titleEquals('User referral types | Drupal');
    $assert_session->responseContains('Default');
    $assert_session->responseContains('Consultant');
    $assert_session->responseContains('Referral Partner');

    $this->drupalLogout();

    $this->drupalLogin($this->consultant_referrer_1);
    $this->drupalGet('admin/structure/user-referral-types');
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalLogout();
  }

  public function testReferralLinksAndAliases() {
    // Visit as consultant referrer 1
    // Test referral link (non-alias)
    $this->drupalGet('user/' . $this->consultant_referrer_1->id() . '/user-referral/' . $this->consultant_referral_type->id());
    $this->assertSession()->statusCodeEquals(200);
    $parsed_url = parse_url($this->getUrl());
    $this->assertEqual($parsed_url['path'], $this->consultant_referral_type->getGotoPath(), t('Referral goto path works'));
    $referral_cookie = $this->getReferralCookie();
    $this->assertReferralCookie($this->consultant_referrer_1->id(), $this->consultant_referral_type->id());

    // Test referral link alias.
    $this->drupalGet($this->consultant_referrer_1->field_referral_id->value);
    $this->assertSession()->statusCodeEquals(200);
    $parsed_url = parse_url($this->getUrl());
    // Esnure redirection worked.
    $this->assertEqual($parsed_url['path'], $this->consultant_referral_type->getGotoPath(), t('Referral goto path works'));

    // Sleep at least one second to ensure the cookie timestamp got extended.
    sleep(1); 
    $this->drupalGet($this->consultant_referrer_1->field_referral_id->value);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertReferralCookie($this->consultant_referrer_1->id(), $this->consultant_referral_type->id());
    $referral_cookie_second = $this->getReferralCookie();
    $this->assertGreaterThan($referral_cookie->timestamp, $referral_cookie_second->timestamp, t('Referral cookie timestamp got extended'));

    // Disable cookie time extending.
    $this->consultant_referral_type = UserReferralType::load($this->consultant_referral_type->id());
    $this->consultant_referral_type->set('reset_cookie_lifetime_for_same_referrer', FALSE);
    $this->consultant_referral_type->save();

    // Test cookie timestamp not changing.
    $this->drupalGet($this->consultant_referrer_1->field_referral_id->value);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertReferralCookie($this->consultant_referrer_1->id(), $this->consultant_referral_type->id());
    $referral_cookie_third = $this->getReferralCookie();
    $this->assertEqual($referral_cookie_second->timestamp, $referral_cookie_third->timestamp, t('Cookie time extension disabling worked'));

    // Test cookie re-assigning.
    $this->drupalGet($this->consultant_referrer_2->field_referral_id->value);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertReferralCookie($this->consultant_referrer_2->id(), $this->consultant_referral_type->id());
    $referral_cookie_fourth = $this->getReferralCookie();
    $this->assertNotEqual($referral_cookie_third->uid, $referral_cookie_fourth->uid, t('Cookie got reassinged'));

    // Disable cookie reassigning.
    $this->consultant_referral_type = UserReferralType::load($this->consultant_referral_type->id());
    $this->consultant_referral_type->set('change_referrer_in_cookie', FALSE);
    $this->consultant_referral_type->save();

    // Test referrer not changing.
    $this->drupalGet($this->consultant_referrer_1->field_referral_id->value);
    $this->assertSession()->statusCodeEquals(200);
    $referral_cookie_fifth = $this->getReferralCookie();
    $this->assertEqual($referral_cookie_fourth->uid, $referral_cookie_fifth->uid, t('Disabling cookie re-assigning works'));

    // Test referral link  alias.
    $this->drupalGet($this->referral_partner_referrer->field_referral_id->value . '/' . $this->referral_partner_referral_type->id());
    $this->assertSession()->statusCodeEquals(200);
    $parsed_url = parse_url($this->getUrl());
    // Esnure redirection worked.
    $this->assertEqual($parsed_url['path'], $this->referral_partner_referral_type->getGotoPath(), t('Referral goto path works'));
    $this->assertReferralCookie($this->referral_partner_referrer->id(), $this->referral_partner_referral_type->id());
    $referral_cookie_sixth = $this->getReferralCookie();
    $this->assertNotEqual($referral_cookie_fifth->uid, $referral_cookie_sixth->uid, t('Cookie got reassinged to different referral type'));

    $config = \Drupal::configFactory()->getEditable('user_referral.settings');

    $config->set('allow_to_change_referral_type', FALSE);
    $config->save();

     // Test referrer not changing.
     $this->drupalGet($this->consultant_referrer_1->field_referral_id->value);
     $this->assertSession()->statusCodeEquals(200);
     $referral_cookie_seventh = $this->getReferralCookie();
     $this->assertEqual($referral_cookie_sixth->uid, $referral_cookie_seventh->uid, t('Disabling cookie re-assigning for diffrent type works'));
  }

}
