<?php

namespace Drupal\user_referral;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\user_referral\Entity\UserReferralType;

/**
 * Form handler for user referral type forms.
 *
 * @internal
 */
class UserReferralTypeForm extends EntityForm {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The Plugin Block Manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * Constructs the NodeTypeForm object.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, BlockManagerInterface $block_manager) {
    $this->entityFieldManager = $entity_field_manager;
    $this->blockManager = $block_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.block')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $type = $this->entity;

    $form['name'] = [
      '#title' => $this->t('Name'),
      '#type' => 'textfield',
      '#default_value' => $type->label(),
      '#description' => $this->t('The human-readable name of this user referral type. This name must be unique.'),
      '#required' => TRUE,
      '#size' => 30,
    ];

    $form['type'] = [
      '#type' => 'machine_name',
      '#default_value' => $type->id(),
      '#maxlength' => EntityTypeInterface::BUNDLE_MAX_LENGTH,
      '#disabled' => $type->isLocked(),
      '#machine_name' => [
        'exists' => ['Drupal\user_referral\Entity\UserReferralType', 'load'],
        'source' => ['name'],
      ],
      '#description' => $this->t('A unique machine-readable name for this user referral type. It must only contain lowercase letters, numbers, and underscores. This name will be used for constructing the referral link URLs, in which underscores will be converted into hyphens.'),
    ];

    $form['description'] = [
      '#title' => $this->t('Description'),
      '#type' => 'textarea',
      '#default_value' => $type->getDescription(),
      '#description' => $this->t('More info about this user referral type'),
    ];

    $field_definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions('user', 'user');
    $referral_field_options = [];
    foreach ($field_definitions as $field_definition) {
      if ($field_definition instanceof FieldConfig && !$field_definition->isDeleted() && $field_definition->get('field_type') == 'referral_id') {
        $referral_field_options[$field_definition->get('field_name')] = $field_definition->label();
      }
    }

    $form['referral_field'] = [
      '#title' => $this->t('Referral Field'),
      '#type' => 'select',
      '#default_value' => $type->getReferralField(),
      '#options' => $referral_field_options,
      '#description' => $this->t('The Referral ID field to be used for this user referral type.'),
      '#required' => TRUE,
    ];

    $form['pattern_container'] = [
      '#type' => 'container',
      '#prefix' => '<div id="pathauto-pattern">',
      '#suffix' => '</div>',
    ];

    $token_types = ['user', 'user_referral_type'];
    $form['pattern_container']['url_alias_pattern'] = [
      '#title' => $this->t('URL alias pattern'),
      '#type' => 'textfield',
      '#default_value' => $type->getURLAliasPattern(),
      '#description' => $this->t('Enter a pattern to be used to generate URL alias for the referral link. Patterns must start with / and must include the token [user_referral_type:referral_field_value] to be valid.  If more than one referral type is used, any non-default referral type must also include /[user_referral_type:type] and the pattern /[user_referral_type:referral_field_value]/[user_referral_type:type] is recommended.'),
      '#required' => FALSE,
      '#size' => 30,
      '#token_types' => $token_types,
      '#min_tokens' => 1,
      '#element_validate' => ['token_element_validate'],
    ];

    // Show the token help relevant to this pattern.
    $form['pattern_container']['token_help'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => $token_types,
    ];

    $form['cookie_lifetime'] = [
      '#title' => $this->t('Cookie lifetime in days'),
      '#type' => 'number',
      '#min' => 1,
      '#max' => 365,
      '#step' => 1,
      '#default_value' => $type->getCookieLifetimeInDays(),
      '#description' => $this->t('Enter number of days the cookie will be valid once a user access referral links of this type.'),
      '#required' => TRUE,
      '#size' => 10,
    ];

    $form['goto_path'] = [
      '#title' => $this->t('Redirect path'),
      '#type' => 'textfield',
      '#default_value' => $type->getGotoPath(),
      '#description' => $this->t('Enter path to which user has to be redirected after accessing referral links of this type.'),
      '#required' => TRUE,
      '#size' => 30,
    ];

    $roles = user_role_names(TRUE);

    $form['roles'] = [
      '#title' => $this->t('Roles'),
      '#type' => 'checkboxes',
      '#default_value' => $type->getRoles(),
      '#options' => $roles,
      '#description' => $this->t('Roles to which this referral type is applicable. You must select at least 1 role to use this feature.'),
      '#size' => 30,
      '#required' => TRUE,
    ];

    $form['reset_cookie_lifetime_for_same_referrer'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow cookie time to be reset to Maximum allowed upon subsequent future visits'),
      '#description' => $this->t('A cookie will be set on user borwser when a referral link is accessed. Enabling this checkbox causes to extend expiry of that cookie in case user takes referral link again from same referrer.'),
      '#default_value' => $type->allowedToExtendCookieLifetimeForSameReferrer(),
    ];

    $form['change_referrer_in_cookie'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow cookie to be reassigned to new Referrers upon subsequent future visits with a new referrer’s URL'),
      '#description' => $this->t('Enabling this checkbox causes to change referrer set in cookie in case user takes referral link again but from a different referrer.'),
      '#default_value' => $type->allowedToChangeReferrer(),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Save user referral type');
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $id = trim($form_state->getValue('type'));
    // '0' is invalid, since elsewhere we check it using empty().
    if ($id == '0') {
      $form_state->setErrorByName('type', $this->t("Invalid machine-readable name. Enter a name other than %invalid.", ['%invalid' => $id]));
    }

    $alias_pattern = $form_state->getValue('url_alias_pattern');
    if (!empty($alias_pattern )) {
      if ($alias_pattern[0] !== '/') {
        $form_state->setErrorByName('url_alias_pattern', $this->t("The path pattern '%path' has to start with a slash.", ['%path' => $alias_pattern]));
      }
      if (strpos($alias_pattern, '[user_referral_type:referral_field_value]') === FALSE) {
        $form_state->setErrorByName('url_alias_pattern', $this->t("The path pattern '%path' has to contain '[user_referral_type:referral_field_value]' token.", ['%path' => $alias_pattern]));
      }
      else {
        $alias_pattern_copy = str_replace('[user_referral_type:type]', $id, $alias_pattern);
        $referrral_types = UserReferralType::loadMultiple();
        foreach ($referrral_types as $type => $referrral_type) {
          if ($id != $type) {
            $other_referral_type_pattern = $referrral_type->getURLAliasPattern();
            $other_referral_type_pattern_copy = str_replace('[user_referral_type:type]', $type, $other_referral_type_pattern);
            if ($form_state->getValue('referral_field') != $referrral_type->getReferralField()) {
              // Different referral fields so, can be removed before comparing.
              $other_referral_type_pattern_copy = str_replace('[user_referral_type:referral_field_value]', '', $other_referral_type_pattern_copy);
            }
            if ($alias_pattern_copy == $other_referral_type_pattern_copy) {
              $form_state->setErrorByName('url_alias_pattern', $this->t("The path pattern '%path' is duplicate of the path pattern of the referral type %type.", ['%path' => $alias_pattern, '%type' => $referrral_type->label()]));
            }
          }
        }
      }
    }

    // Validate the goto page path.
    if (($value = $form_state->getValue('goto_path')) && $value[0] !== '/') {
      $form_state->setErrorByName('goto_path', $this->t("The path '%path' has to start with a slash.", ['%path' => $form_state->getValue('goto_path')]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $type = $this->entity;
    $type->set('type', trim($type->id()));
    $type->set('name', trim($type->label()));

    $status = $type->save();

    $t_args = ['%name' => $type->label()];

    if ($status == SAVED_UPDATED) {
      $this->messenger()->addStatus($this->t('The user referral type %name has been updated.', $t_args));
    }
    elseif ($status == SAVED_NEW) {
      $this->messenger()->addStatus($this->t('The user referral type %name has been added.', $t_args));
      $context = array_merge($t_args, ['link' => $type->toLink($this->t('View'), 'collection')->toString()]);
      $this->logger('node')->notice('Added user referral type %name.', $context);
    }
    $this->entityFieldManager->clearCachedFieldDefinitions();
    $this->blockManager->clearCachedDefinitions();
    $form_state->setRedirectUrl($type->toUrl('collection'));
  }

}
