<?php

namespace Drupal\user_referral;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\user\UserInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides an interface defining a user referral type entity.
 */
interface UserReferralTypeInterface extends ConfigEntityInterface {

  /**
   * Name to be used for the referral cookie.
   */
  const COOKIE_NAME = 'user_referral';

  /**
   * Determines whether the node type is locked.
   *
   * @return string|false
   *   The module name that locks the type or FALSE.
   */
  public function isLocked();

  /**
   * Gets the description.
   *
   * @return string
   *   The description of this referral type.
   */
  public function getDescription();

  /**
   * Gets the referral field.
   *
   * @return string
   *  The referral field for this referral type.
   */
  public function getReferralField();

  /**
   * Gets the URL alias pattern.
   *
   * @return string
   *   The URL alias pattern of this referral type.
   */
  public function getURLAliasPattern();

  /**
   * Gets the cookie lifetime in days.
   *
   * @return int
   *   Number of days the cookie will be valid.
   */
  public function getCookieLifetimeInDays();

  /**
   * Gets the cookie lifetime in seconds.
   *
   * @return int
   *   Number of seconds the cookie will be valid.
   */
  public function getCookieLifetimeInSeconds();

  /**
   * Gets the goto path.
   *
   * @return int
   *   A path user to be redirected after accessing a referral link.
   */
  public function getGotoPath();

  /**
   * Gets the roles.
   *
   * @return array
   *   An array of role names where the referral type applicable.
   */
  public function getRoles();

  /**
   * Get referral link for an account which can be shared.
   *
   * @param $account
   *  User account to which referral link has to be generated against.
   * @param $options
   *  Following options are accepted:
   *    base_path: Provide base path to be used for the link. It nees to include scheme, hostname and base path.
   */
  public function getReferralLink(UserInterface $account, $options = []);

  /**
   * Returns the weight of this referral type (used for sorting).
   *
   * @return int
   *   The referral type weight.
   */
  public function getWeight();

  /**
   * Set cookie on user browser to store info about referral.
   *
   * @param $referrer
   *  Referrer user account
   * @param $request
   *  HTTP Request
   */
  public function setCookie(UserInterface $referrer, Request $request);

}
